package com.fjhx.flow.core;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.fjhx.flow.entity.flow.dto.FlowResult;
import com.fjhx.flow.entity.flow.dto.JumpDto;
import com.fjhx.flow.entity.flow.po.FlowDefinitionLine;
import com.fjhx.flow.entity.flow.po.FlowDefinitionNode;
import com.fjhx.flow.entity.flow.po.FlowExample;
import com.fjhx.flow.entity.flow.po.FlowExampleCurrent;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.flow.enums.HandleTypeEnum;
import com.fjhx.flow.enums.NodeTypeEnum;
import com.fjhx.flow.service.flow.FlowDefinitionLineService;
import com.fjhx.flow.service.flow.FlowDefinitionNodeService;
import com.fjhx.flow.service.flow.FlowExampleCurrentService;
import com.fjhx.flow.service.flow.FlowExampleService;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import lombok.Getter;
import lombok.Setter;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Getter
@Setter
public class FlowJumpContext {

    public static final FlowExampleService flowExampleService = SpringUtil.getBean(FlowExampleService.class);
    public static final FlowDefinitionNodeService flowDefinitionNodeService = SpringUtil.getBean(FlowDefinitionNodeService.class);
    public static final FlowExampleCurrentService flowExampleCurrentService = SpringUtil.getBean(FlowExampleCurrentService.class);
    public static final FlowDefinitionLineService flowDefinitionLineService = SpringUtil.getBean(FlowDefinitionLineService.class);


    public FlowJumpContext(JumpDto dto) {

        jumpDto = dto;

        // 获取流程实例
        flowExample = flowExampleService.getById(dto.getFlowId());
        if (flowExample == null) {
            throw new ServiceException("没有找到流程");
        }
        if (ObjectUtil.notEqual(flowExample.getVersion(), dto.getVersion())) {
            throw new ServiceException("流程已被处理");
        }

        // 获取流程委托对象
        flowDelegate = FlowBean.getBean(flowExample.getFlowKey());

        // 获取流程节点列表
        flowDefinitionNodeList = flowDefinitionNodeService.list(
                q -> q.eq(FlowDefinitionNode::getFlowDefinitionId, flowExample.getDefinitionId()));
        // 获取流程节点连线列表
        flowDefinitionLineList = flowDefinitionLineService.list(
                q -> q.eq(FlowDefinitionLine::getFlowDefinitionId, flowExample.getDefinitionId()));


        // 获取当前审批节点
//        FlowDefinitionNode tempCurrentNode = null;
//        for (FlowDefinitionNode flowDefinitionNode : flowDefinitionNodeList) {
//            if (Objects.equals(flowDefinitionNode.getId(), flowExample.getDefinitionNodeId())) {
//                tempCurrentNode = flowDefinitionNode;
//                break;
//            }
//        }
//        if (tempCurrentNode == null) {
//            throw new ServiceException("没有找到当前审批节点");
//        }
//        currentNode = tempCurrentNode;
        FlowExampleCurrent one = flowExampleCurrentService.getOne(q -> q
                .eq(FlowExampleCurrent::getFlowExampleId, flowExample.getId())
                .eq(FlowExampleCurrent::getHandleUserId, SecurityUtils.getUserId())
        );
        if (ObjectUtil.isEmpty(one)) {
            throw new ServiceException("该流程处理人不是您，您无权审批！");
        }
        currentNode = flowDefinitionNodeService.getById(one.getNodeId());

        // 当前节点类型
        currentNodeType = NodeTypeEnum.getEnum(currentNode.getNodeType());

        // 查找跳转类型
        handleType = HandleTypeEnum.getEnum(dto.getHandleType());

        // 跳转节点用户处理
        selectUserList = dto.getSelectUserList() == null ? new ArrayList<>() : dto.getSelectUserList();

    }

    /**
     * 流程提交参数
     */
    private final JumpDto jumpDto;

    /**
     * 流程实例
     */
    private final FlowExample flowExample;

    /**
     * 流程节点列表
     */
    private final List<FlowDefinitionNode> flowDefinitionNodeList;
    /**
     * 流程节点列表
     */
    private final List<FlowDefinitionLine> flowDefinitionLineList;

    /**
     * 当前审批节点
     */
    private final FlowDefinitionNode currentNode;

    /**
     * 当前审批节点类型
     */
    private final NodeTypeEnum currentNodeType;

    /**
     * 跳转类型
     */
    private final HandleTypeEnum handleType;

    /**
     * 流程委托对象
     */
    private final FlowDelegate flowDelegate;

    //    /**
//     * 跳转节点
//     */
//    private FlowDefinitionNode jumpNode;
    private List<FlowDefinitionNode> jumpNodeList;

    //    /**
//     * 跳转节点处理用户id
//     */
//    private String jumpHandleUserId;
    private List<FlowResult.SelectUser> selectUserList;

    /**
     * 流程当前状态
     */
    private FlowStatusEnum flowStatus;

}
