package com.fjhx.flow.service.flow.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.fjhx.file.utils.ObsFileUtil;
import com.fjhx.flow.core.FlowBean;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.core.FlowJumpContext;
import com.fjhx.flow.core.FlowThreadLocalUtil;
import com.fjhx.flow.entity.flow.dto.FlowResult;
import com.fjhx.flow.entity.flow.dto.InitiateDto;
import com.fjhx.flow.entity.flow.dto.JumpDto;
import com.fjhx.flow.entity.flow.po.*;
import com.fjhx.flow.enums.*;
import com.fjhx.flow.service.flow.*;
import com.fjhx.socket.core.PushParam;
import com.fjhx.socket.core.PushTypeEnum;
import com.fjhx.socket.core.WebSocketPush;
import com.googlecode.aviator.AviatorEvaluator;
import com.googlecode.aviator.Expression;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.entity.SysUserIdentity;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysRoleService;
import com.ruoyi.system.service.ISysUserIdentityService;
import com.ruoyi.system.service.ISysUserService;
import lombok.extern.slf4j.Slf4j;
import net.sf.jsqlparser.JSQLParserException;
import net.sf.jsqlparser.parser.CCJSqlParserUtil;
import net.sf.jsqlparser.statement.select.Select;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.*;
import java.util.function.Function;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

@Slf4j
@Service
public class FlowProcessServiceImpl implements FlowProcessService {

    @Autowired
    private FlowDefinitionService flowDefinitionService;

    @Autowired
    private FlowDefinitionNodeService flowDefinitionNodeService;
    @Autowired
    private FlowDefinitionLineService flowDefinitionLineService;

    @Autowired
    private FlowExampleService flowExampleService;

    @Autowired
    private FlowExampleDetailService flowExampleDetailService;

    @Autowired
    private ISysDeptService sysDeptService;

    @Autowired
    private ISysUserService sysUserService;

    @Autowired
    private FlowInfoService flowInfoService;
    @Autowired
    private ISysUserIdentityService sysUserIdentityService;
    @Autowired
    private FlowExampleCurrentService flowExampleCurrentService;
    @Resource
    private ISysRoleService roleService;

    @DSTransactional
    @Override
    public FlowResult initiate(InitiateDto dto) {
        dto.getData().put("createUser", SecurityUtils.getUserId());

        // 获取流程委托对象
        FlowDelegate flowDelegate = FlowBean.getBean(dto.getFlowKey());

        // 查找可用流程
        FlowDefinition flowDefinition = flowDefinitionService.getOne(q -> q
                .eq(FlowDefinition::getFlowKey, dto.getFlowKey())
                .eq(FlowDefinition::getCurrentVersion, StatusConstant.YES));

        if (flowDefinition == null) {
            throw new ServiceException("流程未配置，请联系管理员");
        }

//        // 流程节点列表
//        List<FlowDefinitionNode> flowDefinitionNodeList1 = flowDefinitionNodeService.list(q -> q
//                .eq(FlowDefinitionNode::getFlowDefinitionId, flowDefinition.getId()));

        // 寻找开始节点
        FlowDefinitionNode startNode = flowDefinitionNodeService.getOne(q -> q
                .eq(FlowDefinitionNode::getFlowDefinitionId, flowDefinition.getId())
                .eq(FlowDefinitionNode::getNodeType, NodeTypeEnum.START.getKey())
        );

        if (startNode == null) {
            throw new ServiceException("流程定义错误：没有找到开始节点");
        }

        // 生成流程id
        long flowId = IdWorker.getId();
        Map<String, Object> templateMap = flowDelegate.initTemplateMap(dto.getData(), dto.getData());

        FlowThreadLocalUtil flowThreadLocal = FlowThreadLocalUtil.create();
        flowThreadLocal.setTemplateData(templateMap);
        flowThreadLocal.setStartData(dto.getData());
        flowThreadLocal.setCurrentData(dto.getData());
        flowThreadLocal.setFlowId(flowId);
        flowThreadLocal.setHandleTypeEnum(HandleTypeEnum.SKIP_TO_NEXT);
        flowThreadLocal.setCurrentNodeTypeEnum(NodeTypeEnum.START);
        flowThreadLocal.setDynamicUserIds(flowDelegate.getHandlingUserList(flowId, dto.getData()));
        flowThreadLocal.setFlowCreateUser(SecurityUtils.getUserId());

        // 寻找下一节点
        List<FlowDefinitionLine> flowLineList = flowDefinitionLineService.list(q -> q
                .eq(FlowDefinitionLine::getFlowDefinitionId, flowDefinition.getId())
                .eq(FlowDefinitionLine::getSourceId, startNode.getId())
        );
        List<FlowDefinitionNode> nextUserNodes = getNextUserNode(startNode, flowLineList);

        FlowDefinitionNode endNode = nextUserNodes.stream()
                .filter(item -> ObjectUtil.equals(NodeTypeEnum.getEnum(item.getNodeType()), NodeTypeEnum.END))
                .findFirst().orElse(null);

        // 流程实例
        FlowExample flowExample = new FlowExample();

        // 如果下一节点不包含结束节点，赋值下一个节点处理用户id
        if (endNode == null) {
            List<FlowResult.SelectUser> selectUserList = dto.getSelectUserList();
            if (selectUserList == null) {
                selectUserList = new ArrayList<>();
            }
            //检查并赋值下一节点处理人信息，以及待办
            FlowResult flowResult = setNextHandleNodeHandleUser(nextUserNodes, selectUserList);
            if (!flowResult.getSuccess()) {
                return flowResult;
            }
            //创建待办信息
            createPendingInfo(nextUserNodes, flowId);

            // 流程进行中
            flowExample.setStatus(FlowStatusEnum.IN_PROGRESS.getKey());
            flowThreadLocal.setFlowStatusEnum(FlowStatusEnum.IN_PROGRESS);
        } else {
            // 流程已通过
            flowExample.setStatus(FlowStatusEnum.PASS.getKey());
            flowThreadLocal.setFlowStatusEnum(FlowStatusEnum.PASS);
        }

        // 执行开始流程方法
        Long businessId;
        try {
            JSONObject startData = dto.getData();
            businessId = flowDelegate.start(flowId, startData);
            //重新赋值模板templateMap
            templateMap = flowDelegate.initTemplateMap(startData, dto.getData());
        } catch (Exception e) {
            log.error("开始节点方法异常", e);
            throw new ServiceException("开始节点方法异常：" + e.getMessage());
        }

        List<FlowExampleDetail> flowExampleDetailList = new ArrayList<>();

        // 开始节点明细
        FlowExampleDetail startExampleDetail = new FlowExampleDetail();
        startExampleDetail.setFlowExampleId(flowId);
        startExampleDetail.setFlowDefinitionNodeId(startNode.getId());
        startExampleDetail.setFlowDefinitionNodeType(startNode.getNodeType());
        startExampleDetail.setHandleType(HandleTypeEnum.SKIP_TO_NEXT.getKey());
        startExampleDetail.setHandleRemark(dto.getRemark());
        startExampleDetail.setSubmitData(JSONObject.toJSONString(dto));
        flowExampleDetailList.add(startExampleDetail);


        // 如果下一节点包含结束节点
        if (endNode != null) {
            FlowDefinitionNode nextUserNode = endNode;


            // 赋值业务id到租户线程
            flowThreadLocal.setBusinessId(businessId);

            // 执行结束方法
            invokeEndMethod(nextUserNode, flowDelegate);

            // 结束流程明细
            FlowExampleDetail endExampleDetail = new FlowExampleDetail();
            endExampleDetail.setFlowExampleId(flowId);
            endExampleDetail.setFlowDefinitionNodeId(nextUserNode.getId());
            endExampleDetail.setFlowDefinitionNodeType(nextUserNode.getNodeType());
            endExampleDetail.setHandleType(HandleTypeEnum.SKIP_TO_NEXT.getKey());
            flowExampleDetailList.add(endExampleDetail);
        }


        flowExample.setId(flowId);
        flowExample.setTitle(templateParse(flowDefinition.getTitleTemplate(), templateMap));
        flowExample.setFlowKey(dto.getFlowKey());
        flowExample.setDefinitionId(flowDefinition.getId());
//        flowExample.setDefinitionNodeId(nextUserNode.getId());
        flowExample.setBusinessId(businessId);
        flowExample.setStartData(dto.getData().toJSONString());
        flowExample.setDefinitionVersion(flowDefinition.getVersionNumber());

        flowExampleService.save(flowExample);
        flowExampleDetailService.saveBatch(flowExampleDetailList);

        // 保存节点附件
        ObsFileUtil.saveFile(dto.getFileList(), startExampleDetail.getId());

        // 推送
        pushInitiateMessage(flowId, flowDefinition, nextUserNodes);

        // 清空ThreadLocal
        FlowThreadLocalUtil.remove();

        return new FlowResult(true);
    }

    @DSTransactional
    @Override
    public FlowResult jump(JumpDto dto) {
        // 获取流程委托对象
        FlowDelegate flowDelegate = FlowBean.getBean(dto.getFlowKey());

        FlowJumpContext context = new FlowJumpContext(dto);
        FlowThreadLocalUtil flowThreadLocal = FlowThreadLocalUtil.create();

        // 发起流程参数
        JSONObject startData = JSONObject.parseObject(context.getFlowExample().getStartData());
        Map<String, Object> templateMap = context.getFlowDelegate().initTemplateMap(dto.getData(), startData);
        flowThreadLocal.setTemplateData(templateMap);
        flowThreadLocal.setStartData(startData);
        flowThreadLocal.setCurrentData(dto.getData());
        flowThreadLocal.setFlowId(dto.getFlowId());
        flowThreadLocal.setBusinessId(context.getFlowExample().getBusinessId());
        flowThreadLocal.setDynamicUserIds(flowDelegate.getHandlingUserList(dto.getFlowId(), startData));
        flowThreadLocal.setFlowCreateUser(context.getFlowExample().getCreateUser());

        FlowResult flowResult;

        // 流程处理逻辑
        switch (context.getHandleType()) {

            // 跳转下一节点
            case SKIP_TO_NEXT:
                flowResult = skipToNext(context);
                break;

            // 结束流程
            case REJECT:
                flowResult = reject(context);
                break;

            // 返回上一节点
            case RETURN_TO_PREVIOUS:
                flowResult = returnToPrevious(context);
                break;

            // 退回到发起人
            case RETURN_TO_SUBMITTER:
                flowResult = returnToSubmitter(context);
                break;

            // 作废
            case CANCELLATION:
                flowResult = cancellation(context);
                break;
            //加签
            case ADD_APPROVAL:
                //修改状态为审批通过
                dto.setHandleType(HandleTypeEnum.SKIP_TO_NEXT.getKey());
                flowResult = transfer(context, dto);
                break;
            //移交
            case TRANSFER:
                flowResult = transfer(context, dto);
                break;
            // 退回到指定节点
            case RETURN_TO_NODE:
                flowResult = returnToNode(context, dto);
                break;

            default:
                throw new ServiceException("未知流程跳转类型");

        }

        // 如果跳转节点审批人有多个，则返回用户列表让当前节点审批人选择下一节点审批人id
        if (!flowResult.getSuccess()) {
            return flowResult;
        }

        //删除当前用户待审批数据,【退回、作废、驳回】删除全部待办
        List<HandleTypeEnum> handleTypeEnums = Arrays.asList(
                HandleTypeEnum.REJECT,
                HandleTypeEnum.RETURN_TO_PREVIOUS,
                HandleTypeEnum.RETURN_TO_SUBMITTER,
                HandleTypeEnum.CANCELLATION,
                HandleTypeEnum.RETURN_TO_NODE
        );
        flowExampleCurrentService.remove(q -> q
                .eq(FlowExampleCurrent::getFlowExampleId, context.getFlowExample().getId())
                .eq(!handleTypeEnums.contains(context.getHandleType()), FlowExampleCurrent::getHandleUserId, SecurityUtils.getUserId())
        );

        flowThreadLocal.setFlowStatusEnum(context.getFlowStatus());
        flowThreadLocal.setHandleTypeEnum(context.getHandleType());
//        flowThreadLocal.setNextHandleUserId(context.getJumpHandleUserId());
        flowThreadLocal.setCurrentNodeTypeEnum(context.getCurrentNodeType());

        //如果非 抄送节点
        if (!NodeHandleTypeEnum.CARBON_COPY.getKey().equals(context.getCurrentNode().getNodeHandleType())) {
            // 执行节点方法
            executiveNodeMethod(context);
        }

        // 保存数据
        editExample(context);

        // 消息推送
        pushJumpMessage(context);

        // 清空ThreadLocal
        FlowThreadLocalUtil.remove();

        return flowResult;
    }

    @Override
    public JSONObject getStartData(Long flowId) {
        if (flowId == null) {
            throw new ServiceException("流程实例id不能为空");
        }
        FlowExample flowExample = flowExampleService.getById(flowId);
        if (flowExample == null) {
            throw new ServiceException("没有找到流程实例");
        }
        return JSONObject.parseObject(flowExample.getStartData());
    }

    private FlowResult setNextHandleNodeHandleUser(List<FlowDefinitionNode> nextUserNodes, List<FlowResult.SelectUser> selectUserList) {
        List<FlowResult.SelectUser> reSelectUserList = new ArrayList<>();

        for (FlowDefinitionNode nextUserNode : nextUserNodes) {
            FlowResult handleUser = getHandleUser(nextUserNode, selectUserList);
            // 如果下一节点处理用户只有1人，赋值用户id
            if (handleUser.getSuccess()) {
                nextUserNode.setJumpHandleUserId(handleUser.getUserId());
            }
            // 如果下一节点处理用户有多人，则返回用户列表让用户选择下一节点处理人id
            else {
                FlowResult.SelectUser selectUser = new FlowResult.SelectUser();
                selectUser.setNodeId(nextUserNode.getId());
                selectUser.setNodeName(nextUserNode.getNodeName());
                selectUser.setUserList(handleUser.getUserList());

                reSelectUserList.add(selectUser);
            }
        }

        //返回选择用户列表
        if (reSelectUserList.size() > 0) {
            FlowResult flowResult = new FlowResult();
            flowResult.setSuccess(false);
            flowResult.setSelectUserList(reSelectUserList);
            return flowResult;
        }

        FlowResult flowResult = new FlowResult();
        flowResult.setSuccess(true);
        return flowResult;
    }

    /**
     * 流程跳转到下一个节点
     */
    private FlowResult skipToNext(FlowJumpContext context) {

        FlowDefinitionNode currentNode = context.getCurrentNode();
        // 寻找跳转节点
        List<FlowDefinitionNode> jumpNodes = getNextUserNode(currentNode, context.getFlowDefinitionLineList());

        //检查当前节点是否全部审批完成
        long count = flowExampleCurrentService.count(q -> q
                .eq(FlowExampleCurrent::getFlowExampleId, context.getFlowExample().getId())
                .eq(FlowExampleCurrent::getNodeId, currentNode.getId())
                .ne(FlowExampleCurrent::getHandleUserId, SecurityUtils.getUserId())
        );

        //如果当前节点没有全部审批完成 || 当前节点处理方式是抄送 直接跳过
        if (count > 0 || NodeHandleTypeEnum.CARBON_COPY.getKey().equals(currentNode.getNodeHandleType())) {
            //回填流程当前状态
            context.setFlowStatus(FlowStatusEnum.getEnum(context.getFlowExample().getStatus()));
            context.setJumpNodeList(new ArrayList<>());
            return new FlowResult(true);
        }

        List<FlowDefinitionNode> newJumpNodes = new ArrayList<>();

        for (FlowDefinitionNode jumpNode : jumpNodes) {
            // 如果下一节点包含结束节点
            if (NodeTypeEnum.END.getKey().equals(jumpNode.getNodeType())) {
                // 流程已通过
                context.setFlowStatus(FlowStatusEnum.PASS);
                context.setJumpNodeList(Collections.singletonList(jumpNode));
                // 流程结束，则流程节点审批人不存在
                jumpNode.setJumpHandleUserId(null);
                return new FlowResult(true);
            }

            // 如果下一节点包含聚合节点
            if (NodeHandleTypeEnum.POLYMERIZE.getKey().equals(jumpNode.getNodeHandleType())) {
                List<FlowDefinitionNode> lastUserNodeList = getLastOneUserNode(jumpNode, context.getFlowDefinitionNodeList(), context.getFlowExample().getDefinitionId());
                List<Long> lastNodeIds = lastUserNodeList.stream().map(FlowDefinitionNode::getId).collect(Collectors.toList());
                long count1 = flowExampleCurrentService.count(q -> q
                        .eq(FlowExampleCurrent::getFlowExampleId, context.getFlowExample().getId())
                        .in(FlowExampleCurrent::getNodeId, lastNodeIds)
                        .ne(FlowExampleCurrent::getHandleUserId, SecurityUtils.getUserId())
                );
                //聚合节点存在未完成的前置节点
                if (count1 > 0) {
                    continue;
                }
            }


            newJumpNodes.add(jumpNode);
        }

        // 寻找节点审批人,并生成待审批数据
//        FlowResult flowResult = getHandleUser(jumpNode, context.getSelectUserList());
        FlowResult flowResult = setNextHandleNodeHandleUser(newJumpNodes, context.getSelectUserList());
        // 流程进行中
        context.setFlowStatus(FlowStatusEnum.IN_PROGRESS);
//        // 赋值流程节点审批人
//        context.setJumpHandleUserId(flowResult.getUserId());
        context.setJumpNodeList(newJumpNodes);

        return flowResult;
    }

    /**
     * 拒绝流程
     */
    private FlowResult reject(FlowJumpContext context) {
        // 流程已驳回
        context.setFlowStatus(FlowStatusEnum.REJECT);
//        // 流程结束，则流程节点审批人不存在
//        context.setJumpHandleUserId(null);

        for (FlowDefinitionNode flowDefinitionNode : context.getFlowDefinitionNodeList()) {
            if (NodeTypeEnum.END.getKey().equals(flowDefinitionNode.getNodeType())) {
                context.setJumpNodeList(new ArrayList<>());
                return new FlowResult(true);
            }
        }
        throw new ServiceException("流程定义错误：未找到结束节点");
    }

    /**
     * 流程跳转到上一个节点
     */
    private FlowResult returnToPrevious(FlowJumpContext context) {

        FlowDefinitionNode currentNode = context.getCurrentNode();

        // 寻找跳转节点
        Long definitionId = context.getFlowExample().getDefinitionId();
        List<FlowDefinitionNode> jumpNodes = getLastOneUserNode(currentNode, context.getFlowDefinitionNodeList(), definitionId);
        context.setJumpNodeList(jumpNodes);

        FlowDefinitionNode startNode = jumpNodes.stream()
                .filter(item -> ObjectUtil.equals(NodeTypeEnum.getEnum(item.getNodeType()), NodeTypeEnum.START))
                .findFirst().orElse(null);

        // 如果流程回退到开始节点
        if (startNode != null) {
            // 流程未发起
            context.setFlowStatus(FlowStatusEnum.READY_START);
            // 流程节点处理人为流程发起人
            startNode.setJumpHandleUserId(String.valueOf(context.getFlowExample().getCreateUser()));

            context.setJumpNodeList(Collections.singletonList(startNode));
            return new FlowResult(true);
        }

        // 赋值流程节点审批人
        FlowResult flowResult = setNextHandleNodeHandleUser(jumpNodes, context.getSelectUserList());
        // 流程进行中
        context.setFlowStatus(FlowStatusEnum.IN_PROGRESS);

        return flowResult;
    }

    /**
     * 退回到发起人
     */
    private FlowResult returnToSubmitter(FlowJumpContext context) {
        // 流程未发起
        context.setFlowStatus(FlowStatusEnum.READY_START);

        for (FlowDefinitionNode flowDefinitionNode : context.getFlowDefinitionNodeList()) {
            if (NodeTypeEnum.START.getKey().equals(flowDefinitionNode.getNodeType())) {
                // 赋值发起人为节点审批人
                flowDefinitionNode.setJumpHandleUserId(String.valueOf(context.getFlowExample().getCreateUser()));

                context.setJumpNodeList(Collections.singletonList(flowDefinitionNode));
                return new FlowResult(true);
            }
        }
        throw new ServiceException("流程定义错误：未找到开始节点");
    }

    /**
     * 流程作废
     */
    private FlowResult cancellation(FlowJumpContext context) {
        // 流程未发起
        context.setFlowStatus(FlowStatusEnum.CANCELLATION);

        for (FlowDefinitionNode flowDefinitionNode : context.getFlowDefinitionNodeList()) {
            if (NodeTypeEnum.END.getKey().equals(flowDefinitionNode.getNodeType())) {
                // 流程作废，则流程节点审批人不存在
                flowDefinitionNode.setJumpHandleUserId(null);

                context.setJumpNodeList(new ArrayList<>());
                return new FlowResult(true);
            }
        }
        throw new ServiceException("流程定义错误：未找到结束节点");
    }

    private FlowResult transfer(FlowJumpContext context, JumpDto dto) {
        //生成下一节点信息（复制当前节点）
        FlowDefinitionNode currentNode = context.getCurrentNode();

        List<FlowResult.SelectUser> handleUserList = dto.getSelectUserList();
        if (ObjectUtil.isEmpty(handleUserList)) {
            List<SysUser> userList = sysUserService.list(Wrappers.<SysUser>query()
                    .eq("company_id", SecurityUtils.getCompanyId())
                    .or().apply("FIND_IN_SET({0}, company_set)", SecurityUtils.getCompanyId())
            );

            FlowResult.SelectUser selectUser = new FlowResult.SelectUser();
            selectUser.setNodeId(currentNode.getId());
            selectUser.setNodeName(currentNode.getNodeName());
            selectUser.setUserList(userList);

            FlowResult flowResult = new FlowResult();
            flowResult.setSuccess(false);
            flowResult.setSelectUserList(Collections.singletonList(selectUser));
            return flowResult;
        }


        // 流程进行中
        context.setFlowStatus(FlowStatusEnum.IN_PROGRESS);

        String handleUserIdStr = handleUserList.stream().map(FlowResult.SelectUser::getHandleUserId).collect(Collectors.joining(","));
        currentNode.setJumpHandleUserId(handleUserIdStr);

        context.setJumpNodeList(Collections.singletonList(currentNode));
        return new FlowResult(true);
    }

    /**
     * 跳转到指定节点
     */
    private FlowResult returnToNode(FlowJumpContext context, JumpDto dto) {
        FlowDefinitionNode currentNode = context.getCurrentNode();
        if (ObjectUtil.isEmpty(dto.getHandleNodeId())) {

            FlowExample flowExample = context.getFlowExample();

            // 获取已完成节点
            List<Long> flowNodeIds = flowExampleDetailService.listObject(FlowExampleDetail::getFlowDefinitionNodeId,
                    q -> q.eq(FlowExampleDetail::getFlowExampleId, dto.getFlowId()));

            // 获取流程全部节点
            List<FlowDefinitionNode> flowDefinitionNodeList = flowDefinitionNodeService.list(
                    q -> q.eq(FlowDefinitionNode::getFlowDefinitionId, flowExample.getDefinitionId())
                            .in(FlowDefinitionNode::getId, flowNodeIds)
                            .ne(FlowDefinitionNode::getId, currentNode.getId())
            );

            FlowResult flowResult = new FlowResult();
            flowResult.setSuccess(false);
            flowResult.setFlowDefinitionNodeList(flowDefinitionNodeList);

            return flowResult;
        }

        Long handleNodeId = dto.getHandleNodeId();
        List<FlowDefinitionNode> flowDefinitionNodeList = context.getFlowDefinitionNodeList();
        // 节点map
        Map<Long, FlowDefinitionNode> nodeMap = flowDefinitionNodeList.stream()
                .collect(Collectors.toMap(FlowDefinitionNode::getId, Function.identity()));

        // 查找指定节点信息
        FlowDefinitionNode jumpNode = nodeMap.get(handleNodeId);
        List<FlowDefinitionNode> jumpNodeList = Collections.singletonList(jumpNode);

        context.setJumpNodeList(jumpNodeList);

        // 如果流程回退到开始节点
        if (NodeTypeEnum.START.getKey().equals(jumpNode.getNodeType())) {
            // 流程未发起
            context.setFlowStatus(FlowStatusEnum.READY_START);
            // 流程节点处理人为流程发起人
            jumpNode.setJumpHandleUserId(String.valueOf(context.getFlowExample().getCreateUser()));
            return new FlowResult(true);
        }

        FlowResult flowResult = setNextHandleNodeHandleUser(jumpNodeList, context.getSelectUserList());
        // 流程进行中
        context.setFlowStatus(FlowStatusEnum.IN_PROGRESS);

        return flowResult;
    }

    /**
     * 执行节点方法
     */
    private void executiveNodeMethod(FlowJumpContext context) {

        // 当前节点方法
        String handlingMethod = context.getCurrentNode().getHandlingMethod();

        // 流程委托对象
        FlowDelegate flowDelegate = context.getFlowDelegate();

        // 流程状态
        FlowStatusEnum flowStatus = context.getFlowStatus();

        // 自定义方法不为空，走自定义方法
        if (StrUtil.isNotBlank(handlingMethod)) {
            try {
                invokeMethod(flowDelegate, handlingMethod);
            } catch (Exception e) {
                if (e instanceof InvocationTargetException) {
                    Throwable targetException = ((InvocationTargetException) e).getTargetException();
                    log.error("跳转节点方法异常", targetException);
                    throw new ServiceException("跳转节点方法异常：" + targetException.getMessage());
                }
                log.error("跳转节点方法异常", e);
                throw new ServiceException("跳转节点方法异常：" + e.getMessage());
            }
        }

        // 执行默认方法
        else {
            try {
                FlowExample flowExample = context.getFlowExample();
                Long flowId = flowExample.getId();
                Long businessId = flowExample.getBusinessId();
                NodeTypeEnum currentNodeType = context.getCurrentNodeType();
                JSONObject data = context.getJumpDto().getData();

                flowDelegate.defaultMethod(flowId, businessId, currentNodeType, flowStatus, data);
            } catch (Exception e) {

                log.error("跳转节点方法异常", e);
                throw new ServiceException("跳转节点方法异常：" + e.getMessage());
            }
        }

        // 结束节点
        if (FlowStatusEnum.PASS.equals(flowStatus)) {
            FlowDefinitionNode endNode = context.getJumpNodeList().stream()
                    .filter(item -> ObjectUtil.equals(NodeTypeEnum.getEnum(item.getNodeType()), NodeTypeEnum.END))
                    .findFirst().orElse(null);
            invokeEndMethod(endNode, flowDelegate);
        }

    }

    /**
     * 执行结束节点方法
     *
     * @param endNode      结束节点
     * @param flowDelegate java委托对象
     */
    private void invokeEndMethod(FlowDefinitionNode endNode, FlowDelegate flowDelegate) {
        try {
            // 执行结束方法
            String handlingMethod = endNode.getHandlingMethod();
            if (StrUtil.isNotBlank(handlingMethod)) {
                // 执行指定方法
                invokeMethod(flowDelegate, handlingMethod);
            } else {
                // 执行默认方法
                flowDelegate.end(
                        FlowThreadLocalUtil.getFlowId(),
                        FlowThreadLocalUtil.getBusinessId(),
                        FlowThreadLocalUtil.getCurrentData());
            }
        } catch (Exception e) {
            if (e instanceof InvocationTargetException) {
                Throwable targetException = ((InvocationTargetException) e).getTargetException();
                log.error("结束节点方法异常", targetException);
                throw new ServiceException("结束节点方法异常：" + targetException.getMessage());
            }
            log.error("结束节点方法异常", e);
            throw new ServiceException("结束节点方法异常：" + e.getMessage());
        }
    }

    /**
     * 执行节点方法
     *
     * @param flowDelegate   流程代理对象
     * @param handlingMethod 流程执行方法名
     */
    private void invokeMethod(FlowDelegate flowDelegate, String handlingMethod) throws Exception {
        Class<? extends FlowDelegate> flowDelegateCls = flowDelegate.getClass();
        Method method = flowDelegateCls.getMethod(handlingMethod.trim());
        method.invoke(flowDelegate);
    }

    /**
     * 保存数据
     */
    private void editExample(FlowJumpContext context) {

        JumpDto dto = context.getJumpDto();
        FlowExample flowExample = context.getFlowExample();
        FlowDefinitionNode currentNode = context.getCurrentNode();
//        FlowDefinitionNode jumpNode = context.getJumpNode();

        //退回发起人后 重新赋值流程标题
        if (ObjectUtil.isNotEmpty(dto.getData())) {
            // 获取流程委托对象
            FlowDelegate flowDelegate = FlowBean.getBean(flowExample.getFlowKey());
            // 查询流程定义
            FlowDefinition flowDefinition = flowDefinitionService.getOne(q -> q
                    .eq(FlowDefinition::getFlowKey, flowExample.getFlowKey())
                    .eq(FlowDefinition::getCurrentVersion, StatusConstant.YES));
            if (flowDefinition == null) {
                throw new ServiceException("流程未配置，请联系管理员");
            }

            //重新赋值流程标题
            Map<String, Object> templateMap = flowDelegate.initTemplateMap(dto.getData(), dto.getData());
            flowExample.setTitle(templateParse(flowDefinition.getTitleTemplate(), templateMap));

            //重新赋值流程发起数据
            flowExample.setStartData(dto.getData().toJSONString());
        }

        // 更新流程实例
//        flowExample.setDefinitionNodeId(jumpNode.getId());
//        flowExample.setHandleUserId(context.getJumpHandleUserId());
        flowExample.setStatus(context.getFlowStatus().getKey());

        //赋值流程结束时间
        if (context.getFlowStatus() == FlowStatusEnum.PASS ||
                context.getFlowStatus() == FlowStatusEnum.REJECT ||
                context.getFlowStatus() == FlowStatusEnum.CANCELLATION) {
            flowExample.setEndTime(new Date());
        }

        boolean updateFlag = flowExampleService.updateById(flowExample);
        if (!updateFlag) {
            throw new ServiceException("流程已被处理");
        }

        // 节点处理明细
        List<FlowExampleDetail> flowExampleDetailList = new ArrayList<>();
        FlowExampleDetail nodeExampleDetail = new FlowExampleDetail();
        nodeExampleDetail.setFlowExampleId(flowExample.getId());
        nodeExampleDetail.setFlowDefinitionNodeId(currentNode.getId());
        nodeExampleDetail.setFlowDefinitionNodeType(currentNode.getNodeType());
        nodeExampleDetail.setHandleType(dto.getHandleType());
        nodeExampleDetail.setHandleRemark(dto.getRemark());
        nodeExampleDetail.setSubmitData(JSONObject.toJSONString(dto));
        flowExampleDetailList.add(nodeExampleDetail);


        List<FlowDefinitionNode> jumpNodeList = context.getJumpNodeList();
        // 如果流程已结束
        if (FlowStatusEnum.PASS.equals(context.getFlowStatus())) {
            for (FlowDefinitionNode flowDefinitionNode : jumpNodeList) {
                if (NodeTypeEnum.END.equals(NodeTypeEnum.getEnum(flowDefinitionNode.getNodeType()))) {
                    FlowExampleDetail endExampleDetail = new FlowExampleDetail();
                    endExampleDetail.setFlowExampleId(flowExample.getId());
                    endExampleDetail.setFlowDefinitionNodeId(flowDefinitionNode.getId());
                    endExampleDetail.setFlowDefinitionNodeType(flowDefinitionNode.getNodeType());
                    endExampleDetail.setHandleType(HandleTypeEnum.SKIP_TO_NEXT.getKey());
                    flowExampleDetailList.add(endExampleDetail);
                }
            }
        } else {
            //保存待办信息
            createPendingInfo(jumpNodeList, context.getFlowExample().getId());
        }

        // 保存流程明细
        flowExampleDetailService.saveBatch(flowExampleDetailList);

        // 保存节点附件
        ObsFileUtil.saveFile(dto.getFileList(), nodeExampleDetail.getId());

    }

    /**
     * 创建待办信息
     */
    void createPendingInfo(List<FlowDefinitionNode> jumpNodeList, Long flowId) {
        List<FlowExampleCurrent> flowExampleCurrentList = new ArrayList<>();
        for (FlowDefinitionNode flowDefinitionNode : jumpNodeList) {
            String jumpHandleUserId = flowDefinitionNode.getJumpHandleUserId();
            List<Long> collect = Arrays.stream(jumpHandleUserId.split(",")).map(Long::valueOf).collect(Collectors.toList());
            for (Long userId : collect) {
                FlowExampleCurrent flowExampleCurrent = new FlowExampleCurrent();
                flowExampleCurrent.setFlowExampleId(flowId);
                flowExampleCurrent.setNodeId(flowDefinitionNode.getId());
                flowExampleCurrent.setHandleUserId(userId);
                flowExampleCurrentList.add(flowExampleCurrent);
            }
        }
        //保存待办信息
        flowExampleCurrentService.saveBatch(flowExampleCurrentList);
    }

    /**
     * 查找上一个节点
     *
     * @param currentNode            当前节点
     * @param flowDefinitionNodeList 流程节点列表
     * @return 用户节点
     */
    private List<FlowDefinitionNode> getLastOneUserNode(FlowDefinitionNode currentNode, List<FlowDefinitionNode> flowDefinitionNodeList, Long flowDefinitionId) {
        List<Long> lastNodeIds = flowDefinitionLineService.listObject(FlowDefinitionLine::getSourceId, q -> q
                .eq(FlowDefinitionLine::getFlowDefinitionId, flowDefinitionId)
                .eq(FlowDefinitionLine::getTargetId, currentNode.getId()));

        // 节点map
        Map<Long, FlowDefinitionNode> nodeMap = flowDefinitionNodeList.stream()
                .collect(Collectors.toMap(FlowDefinitionNode::getId, Function.identity()));

        // 查找上个节点
        List<FlowDefinitionNode> lastNodeList = new ArrayList<>();
        for (Long lastNodeId : lastNodeIds) {
            lastNodeList.add(nodeMap.get(lastNodeId));
        }
//
//        // 如果为分支
//        if (NodeTypeEnum.BRANCH.equals(NodeTypeEnum.getEnum(lastNode.getNodeType()))) {
//            return getLastOneUserNode(lastNode, flowDefinitionNodeList);
//        }

        return lastNodeList;
    }

    /**
     * 查找下一个节点
     *
     * @param currentNode            当前节点
     * @param flowDefinitionLineList 流程节点连线列表
     * @return 用户节点
     */
    private List<FlowDefinitionNode> getNextUserNode(FlowDefinitionNode currentNode, List<FlowDefinitionLine> flowDefinitionLineList) {

        //获取当前节点之后的连线
        List<FlowDefinitionLine> nextLineList = flowDefinitionLineList.stream()
                .filter(item -> ObjectUtil.equals(item.getSourceId(), currentNode.getId()))
                .collect(Collectors.toList());

        List<Long> targetIds = nextLineList.stream().map(FlowDefinitionLine::getTargetId).collect(Collectors.toList());
        Map<Long, FlowDefinitionNode> targetNodeMap = new HashMap<>();
        if (ObjectUtil.isNotEmpty(targetIds)) {
            targetNodeMap = flowDefinitionNodeService.mapKEntity(FlowDefinitionNode::getId, q -> q.in(FlowDefinitionNode::getId, targetIds));
        }

        List<FlowDefinitionNode> nextNodeList = new ArrayList<>();
        for (FlowDefinitionLine flowDefinitionLine : nextLineList) {
            // 获取跳转条件
            String jumpCondition = flowDefinitionLine.getJumpCondition();
            // 跳转条件为空表示默认跳转,或者符合表达式条件
            if (StrUtil.isBlank(jumpCondition) || expressionResult(FlowThreadLocalUtil.getTemplateData(), jumpCondition)) {
                FlowDefinitionNode targetNode = targetNodeMap.get(flowDefinitionLine.getTargetId());
                if (ObjectUtil.isEmpty(flowDefinitionLine)) {
                    throw new ServiceException("目标节点不存在！");
                }
                nextNodeList.add(targetNode);
            }
        }

        return nextNodeList;
    }

    /**
     * 获取处理用户
     *
     * @param node 处理节点
     * @return 处理用户
     */
    private FlowResult getHandleUser(FlowDefinitionNode node, List<FlowResult.SelectUser> handleUserList) {
        Map<Long, String> handleUserMap = handleUserList.stream().collect(Collectors.toMap(FlowResult.SelectUser::getNodeId, FlowResult.SelectUser::getHandleUserId));
        FlowResult flowResult = new FlowResult();

        String handleUserId = handleUserMap.get(node.getId());
        if (ObjectUtil.isNotEmpty(handleUserId)) {
            flowResult.setSuccess(true);
            flowResult.setUserId(handleUserId);
            return flowResult;
        }

        Integer handleObjectType = node.getHandleObjectType();
        List<Long> handleObjectIds = new ArrayList<>();
        Long handleObjectId = null;
        String handleObjectIdStr = node.getHandleObjectId();
        if (ObjectUtil.isNotEmpty(handleObjectIdStr)) {
            handleObjectIds = Arrays.asList(handleObjectIdStr.split(",")).stream()
                    .map(Long::valueOf).collect(Collectors.toList());
            handleObjectId = handleObjectIds.get(0);
        }

        HandleObjectTypeEnum handleObjectTypeEnum = HandleObjectTypeEnum.getEnum(handleObjectType);

        switch (handleObjectTypeEnum) {

            case USER:
                flowResult.setSuccess(true);
                flowResult.setUserId(String.valueOf(handleObjectId));
                return flowResult;

            case DETP_LEADER:
            case DEPT_DIRECTOR:
                SysDept sysDept = sysDeptService.getById(handleObjectId);
                if (sysDept == null) {
                    throw new ServiceException("流程节点处理人异常：没有找到节点处理部门");
                }
                // 部门负责人
                if (HandleObjectTypeEnum.DETP_LEADER.equals(handleObjectTypeEnum)) {
                    Long leaderId = sysDept.getLeaderId();
                    if (leaderId == null) {
                        throw new ServiceException("流程节点处理人异常：没有找到节点处理部门负责人");
                    }
                    flowResult.setUserId(String.valueOf(leaderId));
                }
                // 部门总监
                else {
                    Long directorId = sysDept.getDirectorId();
                    if (directorId == null) {
                        throw new ServiceException("流程节点处理人异常：没有找到节点处理部门总监");
                    }
                    flowResult.setUserId(String.valueOf(directorId));
                }
                flowResult.setSuccess(true);
                return flowResult;

            case POST:
                List<SysUser> postUserList = sysUserService.getListByPostId(handleObjectId);
                if (postUserList.size() == 0) {
                    throw new ServiceException("流程节点处理人异常：节点处理岗位无用户");
                }
                if (postUserList.size() == 1) {
                    flowResult.setSuccess(true);
                    flowResult.setUserId(String.valueOf(postUserList.get(0).getUserId()));
                    return flowResult;
                }
                flowResult.setSuccess(false);
                flowResult.setUserList(postUserList);
                return flowResult;

            case ROLE:
                List<SysUser> roleUserList = sysUserService.getListByRoleIds(handleObjectIds);
                if (roleUserList.size() == 0) {
                    List<SysRole> roleList = roleService.list(Wrappers.<SysRole>query().in("role_id", handleObjectIds));
                    String roleNames = roleList.stream().map(SysRole::getRoleName).collect(Collectors.joining("，"));
                    throw new ServiceException(StrUtil.format("流程节点处理人异常：节点处理角色【{}】无用户", roleNames));
                }
                if (roleUserList.size() == 1) {
                    flowResult.setSuccess(true);
                    flowResult.setUserId(String.valueOf(roleUserList.get(0).getUserId()));
                    return flowResult;
                }
                flowResult.setSuccess(false);
                flowResult.setUserList(roleUserList);
                return flowResult;
            case PARALLEL:
                flowResult.setSuccess(true);
                List<Long> dynamicUserIds = FlowThreadLocalUtil.getDynamicUserIds();
                if (ObjectUtil.isEmpty(dynamicUserIds)) {
                    throw new ServiceException("动态用户列表为空!");
                }
                String dynamicUserIdsStr = dynamicUserIds.stream().map(String::valueOf).collect(Collectors.joining(","));
                flowResult.setUserId(dynamicUserIdsStr);
                return flowResult;
            case DEPT_MANAGER:
                Long deptId;
                if (ObjectUtil.isEmpty(node.getJumpCondition())) {
                    //没表达式取发起人部门
                    SysUser createUser = sysUserService.getById(FlowThreadLocalUtil.getFlowCreateUser());
                    deptId = createUser.getDeptId();
                } else {
                    //有表达式取表达式部门
                    Expression exp = AviatorEvaluator.compile(node.getJumpCondition());
                    Object execute = exp.execute(FlowThreadLocalUtil.getTemplateData());
                    if (ObjectUtil.isEmpty(execute)) {
                        throw new ServiceException("节点用户表达式结果为空！");
                    }
                    deptId = Long.valueOf(execute.toString());
                }
                //递归查询部门
                SysDept sysDept1 = sysDeptService.getById(deptId);
                while (sysDept1 != null && ObjectUtil.notEqual(sysDept1.getType(), 2)) {
                    Long deptParentId = sysDept1.getParentId();
                    if (ObjectUtil.isEmpty(deptParentId)) {
                        sysDept1 = null;
                        break;
                    }
                    sysDept1 = sysDeptService.getById(deptParentId);
                }
                if (sysDept1 == null) {
                    throw new ServiceException("无法找到节点部门信息！");
                }
                List<SysUserIdentity> identityList = sysUserIdentityService.list(Wrappers.<SysUserIdentity>lambdaQuery()
                        .eq(SysUserIdentity::getDeptId, sysDept1.getDeptId())
                        .eq(SysUserIdentity::getIdentity, node.getHandleObjectId())
                );
                String identityName;
                switch (node.getHandleObjectId()) {
                    case "10":
                        identityName = "员工";
                        break;
                    case "15":
                        identityName = "组长";
                        break;
                    case "20":
                        identityName = "主管";
                        break;
                    case "30":
                        identityName = "经理";
                        break;
                    case "40":
                        identityName = "总监";
                        break;
                    default:
                        identityName = "未知职级";
                }
                if (identityList.size() == 0) {
                    throw new ServiceException(StrUtil.format("流程节点处理人异常：部门无部门{}！", identityName));
                }
                List<Long> userIds = identityList.stream().map(SysUserIdentity::getUserId).collect(Collectors.toList());
                List<SysUser> identityUserList = sysUserService.list(Wrappers.<SysUser>lambdaQuery().in(SysUser::getUserId, userIds));
                if (identityList.size() == 1) {
                    flowResult.setSuccess(true);
                    flowResult.setUserId(String.valueOf(identityUserList.get(0).getUserId()));
                    return flowResult;
                }
                flowResult.setSuccess(false);
                flowResult.setUserList(identityUserList);
                return flowResult;
            case BUSINESS_USER:
                Expression exp = AviatorEvaluator.compile(node.getJumpCondition());
                Object execute = exp.execute(FlowThreadLocalUtil.getTemplateData());
                if (ObjectUtil.isEmpty(execute)) {
                    throw new ServiceException("节点用户表达式结果为空！");
                }
                String userId = String.valueOf(execute);
                SysUser sysUser = sysUserService.getById(Long.valueOf(userId));
                if (ObjectUtil.isEmpty(sysUser)) {
                    throw new ServiceException("节点表达式用户不存在！");
                }
                flowResult.setSuccess(true);
                flowResult.setUserId(userId);
                return flowResult;
            case SQL_EXPRESSION:
                String jumpCondition = templateParse(node.getJumpCondition(), FlowThreadLocalUtil.getTemplateData());
                try {
                    if (!(CCJSqlParserUtil.parse(jumpCondition) instanceof Select)) {
                        throw new ServiceException("非法表达式");
                    }
                } catch (JSQLParserException e) {
                    log.error("表达式解析异常：" + e.getMessage(), e);
                    throw new ServiceException("表达式解析失败：" + e.getMessage());
                }
                List<SysUser> userList = sysUserService.list(Wrappers.<SysUser>query().apply("user_id in (" + jumpCondition + ")"));
                if (ObjectUtil.isEmpty(userList)) {
                    throw new ServiceException("节点表达式结果无用户!");
                }
                if (userList.size() == 1) {
                    flowResult.setSuccess(true);
                    flowResult.setUserId(String.valueOf(userList.get(0).getUserId()));
                    return flowResult;
                }
                flowResult.setSuccess(false);
                flowResult.setUserList(userList);
                return flowResult;

            default:
                throw new ServiceException("未知用户处理类型：" + handleObjectType);
        }

    }

    /**
     * 开始节点消息推送
     */
    private void pushInitiateMessage(long flowId, FlowDefinition flowDefinition, List<FlowDefinitionNode> nextNodes) {
        Map<String, Object> businessData = new HashMap<>();
        businessData.put("flowId", flowId);
        businessData.put("flowKey", flowDefinition.getFlowKey());

        FlowDefinitionNode endNode = nextNodes.stream()
                .filter(item -> ObjectUtil.equals(NodeTypeEnum.getEnum(item.getNodeType()), NodeTypeEnum.END))
                .findFirst().orElse(null);

        List<Long> pushUserIds = new ArrayList<>();
        String title;

        //下一节点包含 结束节点
        if (endNode != null) {
            FlowInfo flowInfo = flowInfoService.getById(flowDefinition.getFlowInfoId());
            String flowName = flowInfo == null ? StringPool.EMPTY : flowInfo.getFlowName();

            pushUserIds.add(SecurityUtils.getUserId());
            title = StrUtil.format("您于【{}】发起的【{}】已经审批通过。", DateUtil.formatDateTime(new Date()), flowName);
        } else {
            for (FlowDefinitionNode nextNode : nextNodes) {
                for (String s : nextNode.getJumpHandleUserId().split(",")) {
                    pushUserIds.add(Long.valueOf(s));
                }
            }
            FlowExample flowExample = flowExampleService.getById(flowId);
            title = StrUtil.format("您有一条新的待审批事项，【{}】，请及时处理。", flowExample.getTitle());
        }

        for (Long pushUserId : pushUserIds) {
            WebSocketPush.byUser(PushTypeEnum.MESSAGE, pushUserId, title, 0, businessData);
        }
    }

    /**
     * 跳转节点消息推送
     */
    private void pushJumpMessage(FlowJumpContext context) {
        HandleTypeEnum handleTypeEnum = context.getHandleType();

        // 流程作废不发送消息
        if (Objects.equals(handleTypeEnum, HandleTypeEnum.CANCELLATION)) {
            return;
        }

        FlowExample flowExample = context.getFlowExample();
        FlowStatusEnum flowStatusEnum = context.getFlowStatus();
        FlowDefinitionNode currentNode = context.getCurrentNode();

        //获取所有下一节点处理用户
        List<Long> jumpHandleUserIdList = new ArrayList<>();
        List<FlowDefinitionNode> jumpNodeList = context.getJumpNodeList();
        String jumpHandleUserIds = jumpNodeList.stream()
                .map(FlowDefinitionNode::getJumpHandleUserId).collect(Collectors.joining(","));
        if (ObjectUtil.isNotEmpty(jumpHandleUserIdList)) {
            String[] split = jumpHandleUserIds.split(",");
            for (String s : split) {
                jumpHandleUserIdList.add(Long.valueOf(s));
            }
        }

        Map<String, Object> businessData = new HashMap<>();
        businessData.put("flowId", flowExample.getId());
        businessData.put("flowKey", flowExample.getFlowKey());

        FlowInfo flowInfo = flowInfoService.getOne(q -> q.eq(FlowInfo::getFlowKey, flowExample.getFlowKey()));
        String flowName = flowInfo == null ? StringPool.EMPTY : flowInfo.getFlowName();

        List<PushParam> pushUserList = new ArrayList<>();

        switch (handleTypeEnum) {
            // 跳转下一节点
            case ADD_APPROVAL:
            case TRANSFER:
            case SKIP_TO_NEXT:
                // 如果流程结束
                if (FlowStatusEnum.PASS.equals(flowStatusEnum)) {
                    String title = StrUtil.format("您于【{}】发起的【{}】已经审批通过。",
                            DateUtil.formatDateTime(flowExample.getCreateTime()),
                            flowName);
                    PushParam pushParam = new PushParam();
                    pushParam.setUserId(flowExample.getCreateUser());
                    pushParam.setTitle(title);
                    pushUserList.add(pushParam);
                }
                // 如果下一节点不为结束节点
                else {
                    String title = StrUtil.format("您有一条新的待审批事项，【{}】，请及时处理。", flowExample.getTitle());
                    for (Long userId : jumpHandleUserIdList) {
                        PushParam pushParam = new PushParam();
                        pushParam.setUserId(userId);
                        pushParam.setTitle(title);
                        pushUserList.add(pushParam);
                    }
                }
                break;

            // 驳回流程
            case REJECT:
                String title = StrUtil.format("您于【{}】发起的【{}】在【{}】被驳回。",
                        DateUtil.formatDateTime(flowExample.getCreateTime()),
                        flowName,
                        currentNode.getNodeName());
                PushParam pushParam = new PushParam();
                pushParam.setUserId(flowExample.getCreateUser());
                pushParam.setTitle(title);
                pushUserList.add(pushParam);
                break;

            // 退回上一节点
            case RETURN_TO_NODE:
            case RETURN_TO_PREVIOUS:

                List<Long> lastNodeIds = flowDefinitionLineService.listObject(FlowDefinitionLine::getSourceId, q -> q
                        .eq(FlowDefinitionLine::getFlowDefinitionId, context.getFlowExample().getDefinitionId())
                        .eq(FlowDefinitionLine::getTargetId, context.getCurrentNode().getId()));

                // 查找上次审批节点
                List<FlowExampleDetail> flowExampleDetailList = flowExampleDetailService.list(q -> q
                        .eq(FlowExampleDetail::getFlowExampleId, flowExample.getId())
                        .in(FlowExampleDetail::getFlowDefinitionNodeId, lastNodeIds)
                        .orderByDesc(BaseIdPo::getId));
                for (FlowExampleDetail flowExampleDetail : flowExampleDetailList) {
                    String title1 = StrUtil.format("您于【{}】提交的【{}】被退回，请及时处理。",
                            DateUtil.formatDateTime(flowExampleDetail.getCreateTime()),
                            flowName);
                    PushParam pushParam1 = new PushParam();
                    pushParam1.setUserId(flowExampleDetail.getCreateUser());
                    pushParam1.setTitle(title1);
                    pushUserList.add(pushParam1);
                }
                break;

            // 退回到发起人
            case RETURN_TO_SUBMITTER:
                String title2 = StrUtil.format("您于【{}】提交的【{}】被退回，请及时处理。",
                        DateUtil.formatDateTime(flowExample.getCreateTime()),
                        flowName);

                PushParam pushParam2 = new PushParam();
                pushParam2.setUserId(flowExample.getCreateUser());
                pushParam2.setTitle(title2);
                pushUserList.add(pushParam2);
                break;

            default:
                throw new ServiceException("未知流程跳转类型");
        }

        for (PushParam pushParam : pushUserList) {
            WebSocketPush.byUser(PushTypeEnum.MESSAGE, pushParam.getUserId(), pushParam.getTitle(), 0, businessData);
        }
    }

    // public static void main(String[] args) {
    //     HashMap<String, Object> map = new HashMap<>();
    //     HashMap<Object, Object> dd = new HashMap<>();
    //     dd.put("A", "a1");
    //     dd.put("B", "b1");
    //     dd.put("C", "c1");
    //     map.put("aa", "aa");
    //     map.put("bb", "bb");
    //     map.put("cc", Arrays.asList("A", "B", "C", "D"));
    //     map.put("dd", dd);
    //     System.out.println(expressionResult(map,
    //             "aa == 'aa' && bb == 'bb' && cc[0] == 'A' && dd.A == 'a1' "
    //     ));
    // }

    // public static void main(String[] args) {
    //     System.out.println(
    //             templateParse(
    //                     "测试1：${Integer.parseInt(a)+1};  测试2：${a+1};  测试3：${b.tt};  测试4：${b.vv};  测试5：${b.vv[0]};",
    //                     JSONObject.parseObject("{a: \"1\", b: {tt: \"t\", vv: [\"aa\", \"bb\", \"cc\"] }}"))
    //     );
    // }

    /**
     * 校验el表达式
     */
    public boolean expressionResult(Map<String, Object> map, String expression) {
        try {
            Expression exp = AviatorEvaluator.compile(expression);
            Object execute = exp.execute(map);
            return Boolean.parseBoolean(String.valueOf(execute));
        } catch (Exception e) {
            log.error("el表达式校验错误", e);
            throw new ServiceException("el表达式校验错误：" + e.getMessage());
        }
    }

    /**
     * 模板替换
     *
     * <p>
     * templateStr: ${Integer.parseInt(a)+1};  ${a+1};  ${b.tt};  ${b.vv};  ${b.vv[0]};
     * map:         {a: "1", b: {tt: "t", vv: ["aa", "bb", "cc"] }}
     * return:      2;  11;  t;  ["aa","bb","cc"];  aa;
     *
     * @param templateStr 模板字符串
     * @param map         替换参数
     * @return 替换后的字符串
     */
    private String templateParse(String templateStr, Map<String, Object> map) {
        Pattern pattern = Pattern.compile("\\$\\{(.*?)}");
        Matcher matcher = pattern.matcher(templateStr);
        StringBuffer sr = new StringBuffer();
        while (matcher.find()) {
            String group = matcher.group().replace("$", "");
            Object execute = AviatorEvaluator.compile(group).execute(map);
            if (execute != null) {
                matcher.appendReplacement(sr, execute.toString());
            }
        }
        matcher.appendTail(sr);
        return sr.toString();
    }

}
